@@ -8,6 +8,8 @@ from collections import defaultdict
 
 import numpy as np
 
+from urh.signalprocessing.IQArray import IQArray
+
 DEFAULT_CARRIER_FREQUENCY = 1e3
 DEFAULT_CARRIER_AMPLITUDE = 1
 DEFAULT_CARRIER_PHASE = 0
@@ -87,7 +89,7 @@ def build_modulator_from_args(arguments: argparse.Namespace):
     result.carrier_freq_hz = float(arguments.carrier_frequency)
     result.carrier_amplitude = float(arguments.carrier_amplitude)
     result.carrier_phase_deg = float(arguments.carrier_phase)
-    result.samples_per_bit = int(arguments.bit_length)
+    result.samples_per_symbol = int(arguments.bit_length)
 
     if arguments.modulation_type == "ASK":
         if arguments.parameter_zero.endswith("%"):
@@ -104,7 +106,7 @@ def build_modulator_from_args(arguments: argparse.Namespace):
 
     result.param_for_zero = param_zero
     result.param_for_one = param_one
-    result.modulation_type_str = arguments.modulation_type
+    result.modulation_type = arguments.modulation_type
     result.sample_rate = arguments.sample_rate
 
     return result
@@ -132,6 +134,7 @@ def build_device_from_args(arguments: argparse.Namespace):
                            bandwidth=bandwidth,
                            gain=arguments.gain, if_gain=arguments.if_gain, baseband_gain=arguments.baseband_gain)
 
+    result.freq_correction = arguments.frequency_correction
     if arguments.device_identifier is not None:
         try:
             result.device_number = int(arguments.device_identifier)
@@ -147,11 +150,12 @@ def build_protocol_sniffer_from_args(arguments: argparse.Namespace):
     bh = build_backend_handler_from_args(arguments)
 
     result = ProtocolSniffer(arguments.bit_length, arguments.center, arguments.noise, arguments.tolerance,
-                             Modulator.MODULATION_TYPES.index(arguments.modulation_type),
+                             arguments.modulation_type,
                              arguments.device.lower(), bh)
     result.rcv_device.frequency = arguments.frequency
     result.rcv_device.sample_rate = arguments.sample_rate
     result.rcv_device.bandwidth = arguments.sample_rate if arguments.bandwidth is None else arguments.bandwidth
+    result.rcv_device.freq_correction = arguments.frequency_correction
     if arguments.gain is not None:
         result.rcv_device.gain = arguments.gain
     if arguments.if_gain is not None:
@@ -218,8 +222,8 @@ def modulate_messages(messages, modulator):
         return None
 
     cli_progress_bar(0, len(messages), title="Modulating")
-    nsamples = sum(int(len(msg.encoded_bits) * modulator.samples_per_bit + msg.pause) for msg in messages)
-    buffer = np.zeros(nsamples, dtype=np.complex64)
+    nsamples = sum(int(len(msg.encoded_bits) * modulator.samples_per_symbol + msg.pause) for msg in messages)
+    buffer = IQArray(None, dtype=np.float32, n=nsamples)
     pos = 0
     for i, msg in enumerate(messages):
         # We do not need to modulate the pause extra, as result is already initialized with zeros
@@ -257,7 +261,7 @@ def parse_project_file(file_path: str):
         result["carrier_phase"] = modulator.carrier_phase_deg
         result["parameter_zero"] = modulator.param_for_zero
         result["parameter_one"] = modulator.param_for_one
-        result["modulation_type"] = modulator.modulation_type_str
+        result["modulation_type"] = modulator.modulation_type
 
     return result
 
@@ -280,6 +284,8 @@ def create_parser():
     group1.add_argument("-if", "--if-gain", type=int, help="IF gain to use (only supported for some SDRs)")
     group1.add_argument("-bb", "--baseband-gain", type=int, help="Baseband gain to use (only supported for some SDRs)")
     group1.add_argument("-a", "--adaptive-noise", action="store_true", help="Use adaptive noise when receiving.")
+    group1.add_argument("-fcorr", "--frequency-correction", default=1, type=int,
+                        help="Set the frequency correction for SDR (if supported)")
 
     group2 = parser.add_argument_group('Modulation/Demodulation settings',
                                        "Configure the Modulator/Demodulator. Not required in raw mode."
