@@ -1,5 +1,6 @@
 import math
 import time
+import traceback
 from multiprocessing import Process, Array
 
 import numpy as np
@@ -14,6 +15,7 @@ from urh.controller.dialogs.FilterDialog import FilterDialog
 from urh.controller.dialogs.SendDialog import SendDialog
 from urh.controller.dialogs.SignalDetailsDialog import SignalDetailsDialog
 from urh.signalprocessing.Filter import Filter, FilterType
+from urh.signalprocessing.IQArray import IQArray
 from urh.signalprocessing.ProtocolAnalyzer import ProtocolAnalyzer
 from urh.signalprocessing.Signal import Signal
 from urh.signalprocessing.Spectrogram import Spectrogram
@@ -119,7 +121,6 @@ class SignalFrame(QFrame):
             else:
                 self.ui.lSignalTyp.setText("Complex Signal")
 
-            self.ui.gvLegend.hide()
             self.ui.lineEditSignalName.setText(self.signal.name)
             self.ui.lSamplesInView.setText("{0:,}".format(self.signal.num_samples))
             self.ui.lSamplesTotal.setText("{0:,}".format(self.signal.num_samples))
@@ -132,6 +133,7 @@ class SignalFrame(QFrame):
 
             self.scene_manager = SignalSceneManager(self.signal, self)
             self.ui.gvSignal.scene_manager = self.scene_manager
+            self.scene_manager.scene.setParent(self.ui.gvSignal)
             self.ui.gvSignal.setScene(self.scene_manager.scene)
 
             self.jump_sync = True
@@ -142,7 +144,6 @@ class SignalFrame(QFrame):
             self.set_protocol_visibility()
 
             self.ui.chkBoxShowProtocol.setChecked(True)
-            self.set_qad_tooltip(self.signal.noise_threshold)
             self.ui.btnSaveSignal.hide()
 
             self.show_protocol(refresh=False)
@@ -177,7 +178,7 @@ class SignalFrame(QFrame):
             self.signal.bit_len_changed.connect(self.ui.spinBoxInfoLen.setValue)
             self.signal.qad_center_changed.connect(self.on_signal_qad_center_changed)
             self.signal.noise_threshold_changed.connect(self.on_noise_threshold_changed)
-            self.signal.modulation_type_changed.connect(self.ui.cbModulationType.setCurrentIndex)
+            self.signal.modulation_type_changed.connect(self.ui.cbModulationType.setCurrentText)
             self.signal.tolerance_changed.connect(self.ui.spinBoxTolerance.setValue)
             self.signal.protocol_needs_update.connect(self.refresh_protocol)
             self.signal.data_edited.connect(self.on_signal_data_edited)  # Crop/Delete Mute etc.
@@ -187,7 +188,6 @@ class SignalFrame(QFrame):
             self.signal.saved_status_changed.connect(self.on_signal_data_changed_before_save)
             self.ui.btnSaveSignal.clicked.connect(self.save_signal)
             self.signal.name_changed.connect(self.ui.lineEditSignalName.setText)
-            self.ui.gvLegend.resized.connect(self.on_gv_legend_resized)
 
             self.ui.gvSignal.selection_width_changed.connect(self.start_proto_selection_timer)
             self.ui.gvSignal.sel_area_start_end_changed.connect(self.start_proto_selection_timer)
@@ -210,7 +210,6 @@ class SignalFrame(QFrame):
         self.ui.gvSpectrogram.sel_area_start_end_changed.connect(self.update_selection_area)
         self.ui.gvSpectrogram.selection_height_changed.connect(self.update_number_selected_samples)
         self.ui.gvSignal.sep_area_changed.connect(self.set_qad_center)
-        self.ui.gvSignal.sep_area_moving.connect(self.update_legend)
 
         self.ui.sliderYScale.valueChanged.connect(self.on_slider_y_scale_value_changed)
         self.ui.spinBoxXZoom.valueChanged.connect(self.on_spinbox_x_zoom_value_changed)
@@ -222,7 +221,7 @@ class SignalFrame(QFrame):
         self.proto_selection_timer.timeout.connect(self.update_number_selected_samples)
 
         self.ui.cbSignalView.currentIndexChanged.connect(self.on_cb_signal_view_index_changed)
-        self.ui.cbModulationType.currentIndexChanged.connect(self.on_combobox_modulation_type_index_changed)
+        self.ui.cbModulationType.currentTextChanged.connect(self.on_combobox_modulation_type_text_changed)
         self.ui.cbProtoView.currentIndexChanged.connect(self.on_combo_box_proto_view_index_changed)
 
         self.ui.chkBoxShowProtocol.stateChanged.connect(self.set_protocol_visibility)
@@ -250,8 +249,8 @@ class SignalFrame(QFrame):
         self.ui.spinBoxTolerance.setValue(self.signal.tolerance)
         self.ui.spinBoxCenterOffset.setValue(self.signal.qad_center)
         self.ui.spinBoxInfoLen.setValue(self.signal.bit_len)
-        self.ui.spinBoxNoiseTreshold.setValue(self.signal.noise_threshold)
-        self.ui.cbModulationType.setCurrentIndex(self.signal.modulation_type)
+        self.ui.spinBoxNoiseTreshold.setValue(self.signal.noise_threshold_relative)
+        self.ui.cbModulationType.setCurrentText(self.signal.modulation_type)
         self.ui.btnAdvancedModulationSettings.setVisible(self.ui.cbModulationType.currentText() == "ASK")
 
         self.ui.spinBoxTolerance.blockSignals(False)
@@ -286,7 +285,7 @@ class SignalFrame(QFrame):
             if start < end:
                 max_window_size = 10 ** 5
                 step_size = int(math.ceil((end - start) / max_window_size))
-                power = np.mean(np.abs(self.signal.data[start:end:step_size]))
+                power = np.mean(self.signal.iq_array.subarray(start,end,step_size).magnitudes_normalized)
                 if power > 0:
                     power_str = Formatter.big_value_with_suffix(10 * np.log10(power), 2)
 
@@ -415,17 +414,10 @@ class SignalFrame(QFrame):
             self.save_signal_as()
 
     def save_signal_as(self):
-        if self.signal.filename:
-            initial_name = self.signal.filename
-        else:
-            initial_name = self.signal.name.replace(" ", "-").replace(",", ".").replace(".", "_") + ".complex"
-
-        filename = FileOperator.get_save_file_name(initial_name, wav_only=self.signal.wav_mode)
-        if filename:
-            try:
-                self.signal.save_as(filename)
-            except Exception as e:
-                QMessageBox.critical(self, self.tr("Error saving signal"), e.args[0])
+        try:
+            FileOperator.save_data_dialog(self.signal.name, self.signal.iq_array, self.signal.sample_rate, self.signal.wav_mode)
+        except Exception as e:
+            Errors.exception(e)
 
     def export_demodulated(self):
         try:
@@ -442,16 +434,12 @@ class SignalFrame(QFrame):
                 if filename.endswith(".wav"):
                     data = self.signal.qad.astype(np.float32)
                     data /= np.max(np.abs(data))
-                data = FileOperator.convert_data_to_format(data, filename)
-                FileOperator.save_data(data, filename, self.signal.sample_rate, num_channels=1)
+                FileOperator.save_data(IQArray(data, skip_conversion=True), filename, self.signal.sample_rate, num_channels=1)
                 self.unsetCursor()
             except Exception as e:
                 QMessageBox.critical(self, self.tr("Error exporting demodulated data"), e.args[0])
 
     def draw_signal(self, full_signal=False):
-        gv_legend = self.ui.gvLegend
-        gv_legend.y_sep = -self.signal.qad_center
-
         self.scene_manager.scene_type = self.ui.cbSignalView.currentIndex()
         self.scene_manager.init_scene()
         if full_signal:
@@ -459,13 +447,6 @@ class SignalFrame(QFrame):
         else:
             self.ui.gvSignal.redraw_view()
 
-        legend = LegendScene()
-        legend.setBackgroundBrush(constants.BGCOLOR)
-        legend.setSceneRect(0, self.scene_manager.scene.sceneRect().y(), gv_legend.width(),
-                            self.scene_manager.scene.sceneRect().height())
-        legend.draw_one_zero_arrows(-self.signal.qad_center)
-        gv_legend.setScene(legend)
-
         self.ui.gvSignal.y_sep = -self.signal.qad_center
 
     def restore_protocol_selection(self, sel_start, sel_end, start_message, end_message, old_protoview):
@@ -574,7 +555,8 @@ class SignalFrame(QFrame):
         window_size = 2 ** self.ui.sliderFFTWindowSize.value()
         data_min, data_max = self.ui.sliderSpectrogramMin.value(), self.ui.sliderSpectrogramMax.value()
 
-        redraw_needed = self.ui.gvSpectrogram.scene_manager.set_parameters(self.signal.data, window_size=window_size,
+        redraw_needed = self.ui.gvSpectrogram.scene_manager.set_parameters(self.signal.iq_array.data,
+                                                                           window_size=window_size,
                                                                            data_min=data_min, data_max=data_max)
         self.ui.gvSpectrogram.scene_manager.update_scene_rect()
 
@@ -601,7 +583,6 @@ class SignalFrame(QFrame):
             self.proto_analyzer.eliminate()
             self.ui.gvSignal.scene_manager.eliminate()
 
-        self.ui.gvLegend.eliminate()
         self.ui.gvSignal.eliminate()
         self.ui.gvSpectrogram.eliminate()
 
@@ -646,14 +627,14 @@ class SignalFrame(QFrame):
         start = self.ui.gvSignal.selection_area.x
         end = start + self.ui.gvSignal.selection_area.width
 
-        new_thresh = self.signal.calc_noise_threshold(start, end)
+        new_thresh = self.signal.calc_relative_noise_threshold_from_range(start, end)
         self.ui.spinBoxNoiseTreshold.setValue(new_thresh)
         self.ui.spinBoxNoiseTreshold.editingFinished.emit()
         self.unsetCursor()
 
     @pyqtSlot()
     def on_noise_threshold_changed(self):
-        self.ui.spinBoxNoiseTreshold.setValue(self.signal.noise_threshold)
+        self.ui.spinBoxNoiseTreshold.setValue(self.signal.noise_threshold_relative)
         minimum = self.signal.noise_min_plot
         maximum = self.signal.noise_max_plot
         if self.ui.cbSignalView.currentIndex() == 0:
@@ -688,15 +669,6 @@ class SignalFrame(QFrame):
         self.ui.txtEdProto.setEnabled(True)
         self.ui.txtEdProto.setHtml(self.proto_analyzer.plain_to_html(self.proto_view))
 
-    @pyqtSlot(float)
-    def update_legend(self, y_sep):
-        if self.ui.gvLegend.isVisible():
-            self.ui.gvLegend.y_sep = y_sep
-            self.ui.gvLegend.refresh()
-        self.ui.spinBoxCenterOffset.blockSignals(True)
-        self.ui.spinBoxCenterOffset.setValue(-y_sep)
-        self.ui.spinBoxCenterOffset.blockSignals(False)
-
     @pyqtSlot()
     def handle_protocol_sync_changed(self):
         self.sync_protocol = self.ui.chkBoxSyncSelection.isChecked()
@@ -733,13 +705,6 @@ class SignalFrame(QFrame):
             self.ui.gvSignal.redraw_view(reinitialize=True)
             self.ui.labelRSSI.show()
 
-            if self.ui.cbSignalView.currentIndex() == 1:
-                self.ui.gvLegend.y_scene = self.scene_manager.scene.sceneRect().y()
-                self.ui.gvLegend.scene_height = self.scene_manager.scene.sceneRect().height()
-                self.ui.gvLegend.refresh()
-            else:
-                self.ui.gvLegend.hide()
-
             self.ui.gvSignal.auto_fit_view()
             self.ui.gvSignal.refresh_selection_area()
             qApp.processEvents()
@@ -765,7 +730,7 @@ class SignalFrame(QFrame):
     def on_btn_replay_clicked(self):
         project_manager = self.project_manager
         try:
-            dialog = SendDialog(project_manager, modulated_data=self.signal.data, parent=self)
+            dialog = SendDialog(project_manager, modulated_data=self.signal.iq_array, parent=self)
         except OSError as e:
             logger.error(repr(e))
             return
@@ -994,39 +959,25 @@ class SignalFrame(QFrame):
         self.draw_signal(draw_full_signal)
 
         self.__set_samples_in_view()
-
         self.update_number_selected_samples()
-
-        self.set_qad_tooltip(self.signal.noise_threshold)
         self.on_slider_y_scale_value_changed()
 
     @pyqtSlot(float)
     def on_signal_qad_center_changed(self, qad_center):
         self.ui.gvSignal.y_sep = -qad_center
-        self.ui.gvLegend.y_sep = -qad_center
 
         if self.ui.cbSignalView.currentIndex() > 0:
             self.scene_manager.scene.draw_sep_area(-qad_center)
-            self.ui.gvLegend.refresh()
         self.ui.spinBoxCenterOffset.blockSignals(False)
         self.ui.spinBoxCenterOffset.setValue(qad_center)
 
     def on_spinbox_noise_threshold_editing_finished(self):
-        if self.signal is not None and self.signal.noise_threshold != self.ui.spinBoxNoiseTreshold.value():
+        if self.signal is not None and self.signal.noise_threshold_relative != self.ui.spinBoxNoiseTreshold.value():
             noise_action = ChangeSignalParameter(signal=self.signal, protocol=self.proto_analyzer,
-                                                 parameter_name="noise_threshold",
+                                                 parameter_name="noise_threshold_relative",
                                                  parameter_value=self.ui.spinBoxNoiseTreshold.value())
             self.undo_stack.push(noise_action)
 
-    def set_qad_tooltip(self, noise_threshold):
-        self.ui.cbSignalView.setToolTip(
-            "<html><head/><body><p>Choose the view of your signal: Analog, Demodulated or Spectrogram.</p>"
-            "<p>The quadrature demodulation uses a <b>threshold of magnitude,</b> to <b>supress noise</b>. "
-            "All samples with a magnitude lower than this threshold will be eliminated "
-            "(set to <i>-127</i>) after demodulation.</p>"
-            "<p>Tune this value by selecting a <i>noisy area</i> and mark it as noise using <b>context menu</b>.</p>"
-            "<p>Current noise threshold is: <b>" + str(noise_threshold) + "</b></p></body></html>")
-
     def contextMenuEvent(self, event: QContextMenuEvent):
         if self.signal is None:
             return
@@ -1047,7 +998,7 @@ class SignalFrame(QFrame):
 
     def show_modulation_type(self):
         self.ui.cbModulationType.blockSignals(True)
-        self.ui.cbModulationType.setCurrentIndex(self.signal.modulation_type)
+        self.ui.cbModulationType.setCurrentText(self.signal.modulation_type)
         self.ui.cbModulationType.blockSignals(False)
 
     def on_participant_changed(self):
@@ -1068,20 +1019,17 @@ class SignalFrame(QFrame):
         sdc = SignalDetailsDialog(self.signal, self)
         sdc.show()
 
-    @pyqtSlot(int)
-    def on_combobox_modulation_type_index_changed(self, index: int):
-        if index != self.signal.modulation_type:
+    @pyqtSlot(str)
+    def on_combobox_modulation_type_text_changed(self, txt: str):
+        if txt != self.signal.modulation_type:
             modulation_action = ChangeSignalParameter(signal=self.signal, protocol=self.proto_analyzer,
                                                       parameter_name="modulation_type",
-                                                      parameter_value=index)
+                                                      parameter_value=txt)
 
             self.undo_stack.push(modulation_action)
 
             if self.ui.cbSignalView.currentIndex() == 1:
                 self.scene_manager.init_scene()
-                self.ui.gvLegend.y_scene = self.scene_manager.scene.sceneRect().y()
-                self.ui.gvLegend.scene_height = self.scene_manager.scene.sceneRect().height()
-                self.ui.gvLegend.refresh()
                 self.on_slider_y_scale_value_changed()
 
         self.ui.btnAdvancedModulationSettings.setVisible(self.ui.cbModulationType.currentText() == "ASK")
@@ -1108,13 +1056,6 @@ class SignalFrame(QFrame):
 
         self.ui.lineEditSignalName.setFont(font)
 
-    @pyqtSlot()
-    def on_gv_legend_resized(self):
-        if self.ui.gvLegend.isVisible():
-            self.ui.gvLegend.y_zoom_factor = self.ui.gvSignal.transform().m22()
-            self.ui.gvLegend.refresh()
-            self.ui.gvLegend.translate(0, 1)  # Resize verschiebt sonst Pfeile
-
     @pyqtSlot()
     def on_btn_show_hide_start_end_clicked(self):
         show = self.ui.btnShowHideStartEnd.isChecked()
@@ -1205,7 +1146,7 @@ class SignalFrame(QFrame):
         QApplication.instance().setOverrideCursor(Qt.WaitCursor)
         filter_bw = Filter.read_configured_filter_bw()
         filtered = Array("f", 2 * self.signal.num_samples)
-        p = Process(target=perform_filter, args=(filtered, self.signal.data, f_low, f_high, filter_bw))
+        p = Process(target=perform_filter, args=(filtered, self.signal.iq_array.as_complex64(), f_low, f_high, filter_bw))
         p.daemon = True
         p.start()
 
@@ -1281,7 +1222,6 @@ class SignalFrame(QFrame):
                                                                           filename=filename,
                                                                           include_amplitude=filename.endswith(".fta"))
         except Exception as e:
-            logger.exception(e)
-            Errors.generic_error("Failed to export spectrogram", str(e))
+            Errors.exception(e)
         finally:
             QApplication.restoreOverrideCursor()
